{% extends "!layout.html" %}
{% block sidebartitle %}
    <a href="https://networkx.github.io/">Project Homepage</a> |
    <a href="https://github.com/networkx/networkx">Source Code</a>
    {{ super() }}
{% endblock %}
#{% block document %}
#    {% include "dev_banner.html" %}
#    {{ super() }}
#{% endblock %}
